
# q-expansion of newform 5184.2.a.bi, downloaded from the LMFDB on 08 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 5184
weight = 2
poly_data = [-3, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, 0], [0, 0], [-2, 1], [0, 2], [2, 0], [-1, -2], [4, 1], [-4, 2], [-2, -4], [-6, -1], [4, -4], [-3, -2], [0, -4], [-8, -2], [0, 4], [4, -4], [8, 0], [-7, 2], [-4, -2], [2, 0], [1, 0], [-4, -2], [4, 4], [0, 3], [2, 8], [12, 0], [-4, 4], [-12, -4], [-1, 2], [0, 1], [-4, -2], [-14, -4], [12, -5], [-12, -4], [14, -1], [-8, -4], [5, 6], [0, 0], [-18, 0], [6, -1], [0, 4], [6, 8], [-6, 8], [11, -4], [-6, -5], [-4, 0], [-8, 10], [-4, -2], [6, 12], [11, 2], [8, -1], [-4, 4], [-15, -4], [10, 0], [-8, 11], [-8, 12], [2, 13], [-20, -2], [14, 0], [-8, -7], [-4, 8], [-6, -7], [-12, 4], [12, 0], [3, -12], [-6, 1], [-4, -14], [-6, -8], [18, -4], [6, 0], [16, 8], [-6, -12], [-16, 4], [10, 0], [-4, 12], [-18, 0], [-20, 4], [-11, -6], [-12, 3], [9, -4], [12, -8], [7, 6], [20, -4], [7, -12], [12, 0], [-8, -4], [8, -12], [-31, 4], [4, -12], [40, 0], [12, -16], [2, 8], [-28, 4], [-30, 8], [0, -14], [-12, 8], [12, -12], [-8, 12], [-32, 2], [3, 6], [-4, 8], [-2, -9], [8, 4], [-28, 9], [0, 8], [11, 8], [26, 0], [-16, -9], [12, -4], [7, 8], [-8, 18], [10, 8], [0, -17], [16, 12], [8, 12], [-4, 15], [12, -12], [-10, 16], [12, 12], [-18, -12], [13, -10], [19, -4], [4, 20], [8, 4], [-28, -6], [-6, 17], [-5, 10], [-2, 4], [0, -18], [-26, 8], [-28, 0], [16, -4], [0, -10], [22, -8], [-44, 3], [11, 12], [-30, -1], [16, -2], [-26, -13], [28, -1], [-8, -24], [18, 5], [-20, 0], [-2, -8], [-10, 8], [-30, -4], [-14, 0], [12, -9], [-12, 8], [-26, 4], [-3, 10], [16, -20], [44, 4], [-10, 20], [16, -12], [-26, -12], [-4, -18], [-20, 9], [-9, -24], [6, 7], [-6, -4], [-12, 15], [-8, 10], [-6, 4], [24, -4], [-36, 0], [8, 22], [17, 18], [17, 20], [-44, -4], [0, -20], [-3, -30], [-4, 28], [-35, 0], [-20, 6], [16, 20], [32, -10], [30, 5], [8, 12], [-35, 2], [-8, -12], [10, -12], [3, -2], [24, 20], [0, 0], [-42, 5], [47, 2], [0, 6], [-26, -16], [42, -8], [13, -8], [-36, 8], [-8, 18], [-2, 19], [54, -4], [-32, 8], [-17, 16], [38, -8], [-8, -23], [42, 4], [-34, 17], [-44, -10], [34, 0], [1, 8], [-6, 28], [12, 12], [-8, -10], [-36, -8], [-8, -20], [28, -18], [6, 8], [6, 11], [32, 14], [-2, 16], [-4, 8], [14, 24], [4, 6], [20, 5], [60, -4], [-18, -31], [43, -2], [24, 6], [-36, 5], [40, 4], [-24, 8], [-35, 10], [-24, -12], [-30, -16], [-24, -16], [-4, -28], [-21, -14], [-44, 6], [4, -18], [8, -9], [-12, 28], [-16, -8], [17, 12], [4, -8], [28, 8], [-38, 0], [18, -16], [-16, -2], [32, -12], [2, 8], [0, 12], [2, -20], [-40, 14], [-6, 16], [-56, 4], [-4, 0], [39, 6], [-24, 9], [-40, -8], [1, 0], [-6, -9], [18, -4], [-29, 18], [4, -6], [-6, -19], [-2, 40], [8, -32], [-12, 8], [-25, 6], [6, 0], [8, -24], [-28, -20], [14, -17], [64, -1], [8, -20], [-10, 13], [-67, 6], [8, -18], [58, 0], [20, 8], [25, 8], [52, 12], [0, 24], [-11, 22], [-17, 16], [20, 32], [-32, -24], [-16, 28], [-32, -4], [-11, -10], [8, 4], [4, -20], [-19, -12], [36, 12], [-56, 6], [40, 5], [-20, -28], [42, -20], [-16, -31], [-12, 28], [-63, 10], [-50, -17], [40, 0], [-30, -1], [-30, 8], [-64, -12], [23, -4], [-28, -16], [-8, 14], [-10, -16], [-72, 4], [26, -16], [-30, -8], [22, -32], [-16, -4], [1, -42], [18, 8], [12, -4], [4, 13], [4, 30], [-20, 20], [-22, 8], [12, 0], [-12, -24], [5, 16], [-48, 20], [-56, 20], [-26, 0], [2, 21], [8, -4], [8, 12], [33, 20], [24, -10], [24, -14], [70, 8], [84, -4], [-10, 16], [30, -25], [24, -16], [-30, 8], [12, 10], [-48, 0], [17, -14], [0, 5], [26, -24], [-24, 12], [-67, -6], [0, 20], [-6, -29], [-36, -28], [-62, 13], [-34, -12], [15, 34], [16, 20], [-36, -4], [26, -11], [-72, -6], [-43, -4], [10, 25], [-36, -6], [-71, -14], [40, -11], [52, 12], [46, -8], [-88, 4], [30, 32], [25, -26], [-24, -13], [66, -12], [40, 16], [-8, -32], [-9, 24], [36, 28], [-4, -8], [58, -16], [58, -24], [-48, -2], [20, -32], [82, 9], [-40, 24], [-5, 2], [-54, -8], [12, -16], [31, 0], [-24, 0], [5, 36], [-18, -15], [4, 29], [12, -36], [24, 1], [-56, -14], [-54, -4], [-60, -24], [31, -30], [28, 6], [12, 0], [-11, 16], [-2, 21], [30, 16], [24, -34], [24, 8], [61, 4], [24, 42], [84, 9], [56, -2], [-2, -5], [37, 22], [-96, -4], [-40, 2], [4, 33], [-38, 23], [-28, -2], [-13, 30], [-36, -19], [32, -26], [-24, 28], [-1, -24], [22, -37], [-2, 12], [40, 16], [-3, 56], [10, -37], [36, -28], [-32, 12], [36, 19], [32, -24], [68, 8], [-50, 32], [44, 8], [-8, -40], [19, 28], [2, 51], [-4, 20], [0, -19], [0, 4], [28, 0]]
